@@ -88,8 +88,8 @@ struct _CcRegionPanelPrivate {
         GDBusProxy  *session;
         GCancellable *cancellable;
 
-        GtkWidget *overlay;
-        GtkWidget *notification;
+        GtkWidget *restart_notification;
+        gchar *needs_restart_file_path;
 
         GtkWidget     *language_section;
         GtkListBoxRow *language_row;
@@ -168,6 +168,8 @@ cc_region_panel_finalize (GObject *object)
         g_free (priv->system_language);
         g_free (priv->system_region);
 
+        g_clear_pointer (&priv->needs_restart_file_path, g_free);
+
         chooser = g_object_get_data (G_OBJECT (self), "input-chooser");
         if (chooser)
                 gtk_widget_destroy (chooser);
@@ -213,7 +215,8 @@ restart_now (CcRegionPanel *self)
 {
         CcRegionPanelPrivate *priv = self->priv;
 
-        gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->notification), FALSE);
+        g_file_delete (g_file_new_for_path (priv->needs_restart_file_path),
+                                            NULL, NULL);
 
         g_dbus_proxy_call (priv->session,
                            "Logout",
@@ -223,8 +226,9 @@ restart_now (CcRegionPanel *self)
 }
 
 static void
-show_restart_notification (CcRegionPanel *self,
-                           const gchar   *locale)
+set_restart_notification_visible (CcRegionPanel *self,
+                                  const gchar   *locale,
+                                  gboolean       visible)
 {
 	CcRegionPanelPrivate *priv = self->priv;
         gchar *current_locale = NULL;
@@ -234,20 +238,22 @@ show_restart_notification (CcRegionPanel *self,
                 setlocale (LC_MESSAGES, locale);
         }
 
-        gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification), TRUE);
+        gtk_revealer_set_reveal_child (GTK_REVEALER (priv->restart_notification), visible);
 
         if (locale) {
                 setlocale (LC_MESSAGES, current_locale);
                 g_free (current_locale);
         }
-}
 
-static void
-dismiss_notification (CcRegionPanel *self)
-{
-        CcRegionPanelPrivate *priv = self->priv;
+        if (!visible) {
+                g_file_delete (g_file_new_for_path (priv->needs_restart_file_path),
+                                                    NULL, NULL);
 
-        gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification), FALSE);
+                return;
+        }
+
+        if (!g_file_set_contents (priv->needs_restart_file_path, "", -1, NULL))
+                g_warning ("Unable to create %s", priv->needs_restart_file_path);
 }
 
 typedef struct {
@@ -296,8 +302,13 @@ maybe_notify_finish (GObject      *source,
 
         if (g_str_equal (current_lang_code, target_lang_code) == FALSE ||
             g_str_equal (current_country_code, target_country_code) == FALSE)
-                show_restart_notification (self,
-                                           mnd->category == LC_MESSAGES ? mnd->target_locale : NULL);
+                set_restart_notification_visible (self,
+                                                  mnd->category == LC_MESSAGES ? mnd->target_locale : NULL,
+                                                  TRUE);
+        else
+                set_restart_notification_visible (self,
+                                                  mnd->category == LC_MESSAGES ? mnd->target_locale : NULL,
+                                                  FALSE);
 out:
         g_free (target_country_code);
         g_free (target_lang_code);
@@ -657,6 +668,10 @@ setup_language_section (CcRegionPanel *self)
         priv->formats_row = GTK_LIST_BOX_ROW (WID ("formats_row"));
         priv->formats_label = WID ("formats_label");
 
+        priv->restart_notification = WID ("restart-revealer");
+        widget = WID ("restart-button");
+        g_signal_connect_swapped (widget, "clicked", G_CALLBACK (restart_now), self);
+
         widget = WID ("language_list");
         gtk_list_box_set_selection_mode (GTK_LIST_BOX (widget),
                                          GTK_SELECTION_NONE);
@@ -843,8 +858,8 @@ add_input_row (CcRegionPanel   *self,
         gtk_widget_set_halign (label, GTK_ALIGN_START);
         gtk_widget_set_margin_start (label, 20);
         gtk_widget_set_margin_end (label, 20);
-        gtk_widget_set_margin_top (label, 12);
-        gtk_widget_set_margin_bottom (label, 12);
+        gtk_widget_set_margin_top (label, 18);
+        gtk_widget_set_margin_bottom (label, 18);
         gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
 
         if (strcmp (type, INPUT_SOURCE_TYPE_IBUS) == 0) {
@@ -1815,7 +1830,6 @@ static void
 cc_region_panel_init (CcRegionPanel *self)
 {
 	CcRegionPanelPrivate *priv;
-        GtkWidget *button;
 	GError *error = NULL;
 
 	priv = self->priv = REGION_PANEL_PRIVATE (self);
@@ -1848,18 +1862,16 @@ cc_region_panel_init (CcRegionPanel *self)
                                   session_proxy_ready,
                                   self);
 
-        priv->notification = GTK_WIDGET (gtk_builder_get_object (priv->builder, "notification"));
-
-        button = GTK_WIDGET (gtk_builder_get_object (priv->builder, "restart-button"));
-        g_signal_connect_swapped (button, "clicked", G_CALLBACK (restart_now), self);
-
-        button = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dismiss-button"));
-        g_signal_connect_swapped (button, "clicked", G_CALLBACK (dismiss_notification), self);
-
         setup_login_button (self);
         setup_language_section (self);
         setup_input_section (self);
 
-        priv->overlay = GTK_WIDGET (gtk_builder_get_object (priv->builder, "overlay"));
-	gtk_container_add (GTK_CONTAINER (self), priv->overlay);
+	gtk_container_add (GTK_CONTAINER (self),
+                           GTK_WIDGET (gtk_builder_get_object (priv->builder, "vbox_region")));
+
+        priv->needs_restart_file_path = g_build_filename (g_get_user_runtime_dir (),
+                                                          "gnome-control-center-region-needs-restart",
+                                                          NULL);
+        if (g_file_query_exists (g_file_new_for_path (priv->needs_restart_file_path), NULL))
+                set_restart_notification_visible (self, NULL, TRUE);
 }
