@@ -39,10 +39,14 @@ extern GType cc_date_time_panel_get_type (void);
 extern GType cc_display_panel_get_type (void);
 #endif
 extern GType cc_info_panel_get_type (void);
+extern GType cc_info_overview_panel_get_type (void);
+extern GType cc_info_default_apps_panel_get_type (void);
+extern GType cc_info_removable_media_panel_get_type (void);
 extern GType cc_keyboard_panel_get_type (void);
 extern GType cc_mouse_panel_get_type (void);
 #ifdef BUILD_NETWORK
 extern GType cc_network_panel_get_type (void);
+extern GType cc_wifi_panel_get_type (void);
 #endif /* BUILD_NETWORK */
 extern GType cc_notifications_panel_get_type (void);
 extern GType cc_goa_panel_get_type (void);
@@ -87,10 +91,14 @@ static struct {
   PANEL_TYPE("display",          cc_display_panel_get_type      ),
 #endif
   PANEL_TYPE("info",             cc_info_panel_get_type         ),
+  PANEL_TYPE("info-overview",    cc_info_overview_panel_get_type),
+  PANEL_TYPE("default-apps",     cc_info_default_apps_panel_get_type),
+  PANEL_TYPE("removable-media",  cc_info_removable_media_panel_get_type),
   PANEL_TYPE("keyboard",         cc_keyboard_panel_get_type     ),
   PANEL_TYPE("mouse",            cc_mouse_panel_get_type        ),
 #ifdef BUILD_NETWORK
   PANEL_TYPE("network",          cc_network_panel_get_type      ),
+  PANEL_TYPE("wifi",             cc_wifi_panel_get_type         ),
 #endif
   PANEL_TYPE("notifications",    cc_notifications_panel_get_type),
   PANEL_TYPE("online-accounts",  cc_goa_panel_get_type          ),
@@ -127,9 +135,9 @@ cc_panel_loader_get_panels (void)
 static int
 parse_categories (GDesktopAppInfo *app)
 {
-  const char *categories;
-  char **split;
-  int retval;
+  GStrv split;
+  const gchar *categories;
+  gint retval;
 
   categories = g_desktop_app_info_get_categories (app);
   split = g_strsplit (categories, ";", -1);
@@ -139,7 +147,18 @@ parse_categories (GDesktopAppInfo *app)
 #define const_strv(s) ((const gchar* const*) s)
 
 #ifdef CC_ENABLE_ALT_CATEGORIES
-  if (g_strv_contains (const_strv (split), "X-GNOME-ConnectivitySettings"))
+  if (g_strv_contains (const_strv (split), "X-GNOME-Hidden"))
+    retval = CC_CATEGORY_HIDDEN;
+  else if (g_strv_contains (const_strv (split), "HardwareSettings"))
+    retval = CC_CATEGORY_HARDWARE;
+  else if (g_strv_contains (const_strv (split), "X-GNOME-PersonalSettings"))
+    retval = CC_CATEGORY_PERSONAL;
+  else if (g_strv_contains (const_strv (split), "X-GNOME-SystemSettings"))
+    retval = CC_CATEGORY_SYSTEM;
+#else
+  if (g_strv_contains (const_strv (split), "X-GNOME-AltHidden"))
+    retval = CC_CATEGORY_HIDDEN;
+  else if (g_strv_contains (const_strv (split), "X-GNOME-ConnectivitySettings"))
     retval = CC_CATEGORY_CONNECTIVITY;
   else if (g_strv_contains (const_strv (split), "X-GNOME-PersonalizationSettings"))
     retval = CC_CATEGORY_PERSONALIZATION;
@@ -151,13 +170,6 @@ parse_categories (GDesktopAppInfo *app)
     retval = CC_CATEGORY_DETAILS;
   else if (g_strv_contains (const_strv (split), "HardwareSettings"))
     retval = CC_CATEGORY_HARDWARE;
-#else
-  if (g_strv_contains (const_strv (split), "HardwareSettings"))
-    retval = CC_CATEGORY_HARDWARE;
-  else if (g_strv_contains (const_strv (split), "X-GNOME-PersonalSettings"))
-    retval = CC_CATEGORY_PERSONAL;
-  else if (g_strv_contains (const_strv (split), "X-GNOME-SystemSettings"))
-    retval = CC_CATEGORY_SYSTEM;
 #endif
 
 #undef const_strv
@@ -175,23 +187,21 @@ parse_categories (GDesktopAppInfo *app)
 void
 cc_panel_loader_fill_model (CcShellModel *model)
 {
-  int i;
+  guint i;
 
   for (i = 0; i < G_N_ELEMENTS (all_panels); i++)
     {
-      GDesktopAppInfo *app;
-      char *desktop_name;
-      int category;
+      g_autoptr (GDesktopAppInfo) app;
+      g_autofree gchar *desktop_name = NULL;
+      gint category;
 
       desktop_name = g_strconcat ("gnome-", all_panels[i].name,
                                   "-panel.desktop", NULL);
       app = g_desktop_app_info_new (desktop_name);
-      g_free (desktop_name);
 
-      if (app == NULL)
+      if (!app)
         {
-          g_warning ("Ignoring broken panel %s (missing desktop file)",
-                     all_panels[i].name);
+          g_warning ("Ignoring broken panel %s (missing desktop file)", all_panels[i].name);
           continue;
         }
 
@@ -199,8 +209,15 @@ cc_panel_loader_fill_model (CcShellModel *model)
       if (G_UNLIKELY (category < 0))
         continue;
 
-      cc_shell_model_add_item (model, category, G_APP_INFO (app), all_panels[i].name);
-      g_object_unref (app);
+      /* Consult OnlyShowIn/NotShowIn for desktop environments */
+      if (!g_desktop_app_info_get_show_in (app, NULL))
+        continue;
+
+      /* Only add the panel when it is not hidden, e.g. the Details subpanels
+       * that are only visible in the new Shell.
+       */
+      if (category != CC_CATEGORY_HIDDEN)
+        cc_shell_model_add_item (model, category, G_APP_INFO (app), all_panels[i].name);
     }
 }
 
